package com.narvii.wallet

import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.net.Uri
import android.support.annotation.IdRes
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.animation.Animation
import android.view.animation.RotateAnimation
import android.widget.*
import com.narvii.amino.R
import com.narvii.model.IBaseProduct
import com.narvii.model.api.ApiResponse
import com.narvii.monetization.utils.ClaimGiftDialog
import com.narvii.monetization.utils.ClaimGiftHintLayout
import com.narvii.util.Utils
import com.narvii.util.http.ApiRequest
import com.narvii.util.http.ApiResponseListener
import com.narvii.util.http.ApiService
import com.narvii.util.http.NameValuePair
import com.narvii.util.text.TextUtils

/**
 * Created by troy on 2018/7/22.
 */
class RedeemCouponComponent : RelativeLayout {
    private val COUPON_STATUS_NO_COUPON_AVAILABLE = 1
    private val COUPON_STATUS_COUPON_TO_CLAIM = 2
    private val COUPON_STATUS_AVAILABLE_COUPON = 3

    private var apiService: ApiService
    private var membershipService: MembershipService

    private val earnFreeCoins by bind<TextView>(R.id.earn_coins_text)
    private val redeemCoinCount by bind<TextView>(R.id.redeem_coin_count)
    private val redeemText by bind<TextView>(R.id.redeem_text)
    private val redeemAutoRenewCheck by bind<CheckBox>(R.id.redeem_auto_renew)
    private val couponDetail by bind<TextView>(R.id.coupon_detail)
    private val redeemButton by bind<LinearLayout>(R.id.redeem)
    private val couponHint by bind<ClaimGiftHintLayout>(R.id.claim_coupon_hint)
    private val couponApplyCheckbox by bind<CheckBox>(R.id.apply_coupon_check_box)
    private val couponSectionDivider by bind<View>(R.id.coupon_selection_divider)
    private val purchaseLoading by bind<ImageView>(R.id.purchase_loading)
    private val purchaseLoadingAnimation by lazy { lazyInitPurchaseLoading() }
    private val claimDialog by lazy { lazyInitClaimDialog() }
    private var callback: IRedeemCouponCallback? = null
    private var toRedeemProduct: IBaseProduct? = null
    private var suggestedCoupon: Coupon? = null
    private var couponToUse: Coupon? = null
    private var couponList: ArrayList<Coupon>? = ArrayList()
    private var isCouponFetchingInProcess: Boolean = false

    constructor(context: Context?) : super(context)

    constructor(context: Context?, attributes: AttributeSet) : super(context, attributes)

    private fun<T: View> RedeemCouponComponent.bind(@IdRes res: Int): Lazy<T> {
        return kotlin.lazy(kotlin.LazyThreadSafetyMode.NONE) { findViewById<T>(res) as T }
    }

    init {
        LayoutInflater.from(context).inflate(R.layout.component_redeem_coupon, this, true)
        apiService = Utils.getNVContext(context).getService("api")
        membershipService = Utils.getNVContext(context).getService("membership")
    }

    override fun onFinishInflate() {
        super.onFinishInflate()
        setOnClickListener {  }
        earnFreeCoins.setOnClickListener {
            val intent = Intent(Intent.ACTION_VIEW, Uri.parse("ndc://wallet"))
            context.startActivity(intent)
        }
        redeemButton.setOnClickListener {
            purchaseLoading.visibility = View.VISIBLE
            purchaseLoading.startAnimation(purchaseLoadingAnimation)
            redeemText.text = context.getString(R.string.purchasing)
            redeemCoinCount.visibility = View.GONE
            callback?.onRedeemRequested(toRedeemProduct, couponToUse)
        }
        couponApplyCheckbox.setOnCheckedChangeListener { buttonView, isChecked ->
            if (suggestedCoupon != null) {
                updateCouponSection(COUPON_STATUS_AVAILABLE_COUPON, isChecked)
                updateRedeemButtonText()
            }
        }
        couponHint.setOnClickListener {
            claimDialog.show(membershipService.claimCoupon, false)
        }
        fetchCouponList()
    }



    fun bindCoupons(couponList: ArrayList<Coupon>?) {
        if (couponList != null) {
            //Only coupons whose value are lower than the item price are available
            for (coupon in couponList) {
                val productPrice : Int = if(toRedeemProduct == null) 0 else toRedeemProduct!!.getProductPrice()
                coupon.hasProperValue = coupon.getValue() <= productPrice
            }
        }
        if (membershipService.canGetNewMemberRewards()) {
            suggestedCoupon = null
            updateCouponSection(COUPON_STATUS_COUPON_TO_CLAIM, false)
        } else {
            if (couponList == null || couponList.isEmpty()) {
                suggestedCoupon = null
                updateCouponSection(COUPON_STATUS_NO_COUPON_AVAILABLE, false)
            } else {
                for (coupon in couponList) {
                    if (coupon.isAvailable()) {
                        suggestedCoupon = coupon
                        break
                    }
                }
                if (suggestedCoupon == null) {
                    updateCouponSection(COUPON_STATUS_NO_COUPON_AVAILABLE, false)
                } else {
                    updateCouponSection(COUPON_STATUS_AVAILABLE_COUPON,
                            suggestedCoupon!!.hasProperValue && couponApplyCheckbox.isChecked)
                }
            }
        }

        updateRedeemButtonText()
    }

    fun bindProduct(product: IBaseProduct, isForRedeem: Boolean, callback: IRedeemCouponCallback?) {
        this.callback = callback
        this.toRedeemProduct = product
        purchaseLoadingAnimation.cancel()
        purchaseLoading.clearAnimation()
        purchaseLoading.visibility = View.GONE
        redeemText.text = context.getString(if (isForRedeem) R.string.membership_redeem else R.string.buy_with)
        redeemCoinCount.visibility = View.VISIBLE
        updateRedeemButtonText()
        updateEarnFreeCoinsContent()
        fetchCouponList()
    }

    fun hasAutoRenewCheckbox(hasAutoRenew: Boolean) {
        redeemAutoRenewCheck.visibility = if(hasAutoRenew) View.VISIBLE else View.GONE
    }

    fun isAutoRenew(): Boolean {
        return redeemAutoRenewCheck.isChecked
    }

    fun destroy() {
        purchaseLoadingAnimation.cancel()
        purchaseLoading.clearAnimation()
    }

    private fun lazyInitPurchaseLoading(): Animation {
        val animation = RotateAnimation(0f, 360f, Animation.RELATIVE_TO_SELF, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f)
        animation.repeatCount = Animation.INFINITE
        animation.duration = 1000
        return animation
    }

    private fun lazyInitClaimDialog(): ClaimGiftDialog {
        val dialog = ClaimGiftDialog(Utils.getNVContext(context))
        dialog.source = "Modal"
        dialog.setOnDismissListener { fetchCouponList() }
        return dialog
    }

    private fun fetchCouponList() {
        if (isCouponFetchingInProcess) {
            bindCoupons(couponList)
            return
        }

        isCouponFetchingInProcess = true
        val req = ApiRequest.builder().path("/coupon/new-user-coupon").build()
        apiService.exec(req, object : ApiResponseListener<CouponListResponse>(CouponListResponse::class.java){
            override fun onFinish(req: ApiRequest?, resp: CouponListResponse?) {
                super.onFinish(req, resp)
                isCouponFetchingInProcess = false
                couponList = resp?.couponList
                bindCoupons(couponList)
            }

            override fun onFail(req: ApiRequest?, statusCode: Int, headers: MutableList<NameValuePair>?, message: String?, resp: ApiResponse?, t: Throwable?) {
                super.onFail(req, statusCode, headers, message, resp, t)
                isCouponFetchingInProcess = false
            }
        })
    }

    private fun updateRedeemButtonText() {
        if (toRedeemProduct != null && suggestedCoupon != null && couponApplyCheckbox.isChecked) {
            val finalPrice = toRedeemProduct!!.getProductPrice() - suggestedCoupon!!.getValue()
            redeemCoinCount.text = (if (finalPrice >= 0) finalPrice else 0).toString()
        } else if (toRedeemProduct != null) {
            redeemCoinCount.text = toRedeemProduct!!.getProductPrice().toString()
        }
    }

    private fun updateCouponSection(couponStatus: Int, couponSelected: Boolean) {
        when (couponStatus) {
            COUPON_STATUS_NO_COUPON_AVAILABLE -> {
                couponToUse = null
                couponDetail.visibility = View.GONE
                couponHint.visibility = View.GONE
                couponApplyCheckbox.visibility = View.GONE
                couponSectionDivider.visibility = View.GONE
            }
            COUPON_STATUS_COUPON_TO_CLAIM -> {
                couponToUse = null
                couponDetail.visibility = View.VISIBLE
                couponHint.visibility = View.VISIBLE
                couponApplyCheckbox.visibility = View.GONE
                couponSectionDivider.visibility = View.VISIBLE
                couponDetail.text = resources.getString(R.string.coupons_title)
            }
            COUPON_STATUS_AVAILABLE_COUPON -> {
                couponDetail.visibility = View.VISIBLE
                couponHint.visibility = View.GONE
                couponApplyCheckbox.visibility = View.VISIBLE
                couponSectionDivider.visibility = View.VISIBLE
                couponApplyCheckbox.isChecked = couponSelected
                couponToUse = if (couponSelected) suggestedCoupon else null
                couponDetail.text = resources.getString(
                        (if (couponSelected) R.string.coupons_title_with_discount else R.string.one_coupon_available_hint),
                        suggestedCoupon?.getValue())
                if (couponSelected) {
                    val content = resources.getString(R.string.coupons_title_with_discount, suggestedCoupon?.getValue())
                    val sb = SpannableStringBuilder(content)
                    sb.setSpan(ForegroundColorSpan(Color.parseColor("#1AD422")), content.indexOf('-'), content.length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
                    couponDetail.text = sb
                } else {
                    couponDetail.text = resources.getString(R.string.one_coupon_available_hint, suggestedCoupon?.getValue())
                }
            }
        }
    }

    private fun updateEarnFreeCoinsContent() {
        val ownedCoinsText = context.getString(R.string.membership_owned_coins, TextUtils.numberFormat.format(membershipService.walletBalance()))
        val earnFreeCoinsText = context.getString(R.string.membership_earn_coins)
        val content = ownedCoinsText + " " + earnFreeCoinsText
        val sb = SpannableStringBuilder(content)
        sb.setSpan(ForegroundColorSpan(Color.parseColor("#B9B9B9")), 0, ownedCoinsText.length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        sb.setSpan(ForegroundColorSpan(Color.parseColor("#4A90E2")), ownedCoinsText.length, content.length, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        earnFreeCoins.text = sb
    }

    interface IRedeemCouponCallback {
        fun onRedeemRequested(product: IBaseProduct?, coupon: Coupon?)
    }
}